{% load cms_tags allink_cms_tags sekizai_tags i18n thumbnail allink_image_tags allink_link_tags %}

<div class="teaser teaser--list">

    <div class="teaser__col-container">

        <div class="teaser__col teaser__col-1">
            {% include 'allink_teaser/includes/_image.html' with teaser_image=teaser_image %}
        </div>

        <div class="teaser__col teaser__col-2">
            {% include 'allink_teaser/includes/_technical-title.html' with teaser_technical_title=teaser_technical_title %}
            {% include 'allink_teaser/includes/_title.html' with teaser_title=teaser_title %}
            {% include 'allink_teaser/includes/_description.html' with teaser_description=teaser_description %}
            {% include 'allink_teaser/includes/_link.html' with teaser_link=teaser_link teaser_link_text=teaser_link_text %}
        </div>

    </div>

</div>
